#ifndef FrameCPP__COMMON__FRAME_BUFFER_INTERFACE_HH
#define FrameCPP__COMMON__FRAME_BUFFER_INTERFACE_HH

#if !defined(SWIGIMPORTED)
#include <iostream>
#endif /* !defined(SWIGIMPORTED) */

#include "general/types.hh"

#include "framecpp/Common/StreamFilter.hh"

namespace FrameCPP
{
  namespace Common
  {
    //===================================================================
    // \brief Interface definition to Frame Buffer
    //===================================================================
    class FrameBufferInterface
    {
    public:
      typedef std::streambuf::char_type	char_type;
      typedef INT_4U buffer_size_type;

      /// \brief The default size for buffered input and output.
      static const buffer_size_type	M_BUFFER_SIZE_DEFAULT;
      static const buffer_size_type	M_BUFFER_SIZE_SYSTEM = ~buffer_size_type( 0 );

      FrameBufferInterface( );

      virtual ~FrameBufferInterface( );

      bool AutoDelete( ) const;

      void AutoDelete( bool Value );

      virtual void FilterAdd( FrameCPP::Common::StreamFilter* Filter ) = 0;
      virtual void FilterRemove( FrameCPP::Common::StreamFilter* Filter ) = 0;
      virtual bool FilterInternally( ) const = 0;
      virtual void buffer( ) = 0;
      template< class DT >
      void filter( const DT& Data );
      virtual void filter( const char_type* Start,
			   const char_type* End ) = 0;

      //-----------------------------------------------------------------
      /// \brief Retrieve the string identification of the buffer.
      ///
      /// \return
      ///     An identifier associated with the open buffer.
      //-----------------------------------------------------------------
      const std::string& BufferId( ) const;

    protected:
      void bufferId( const std::string& Id );

    private:
      bool		m_auto_delete;

      std::string	m_stream_id;
    };
    
    inline bool FrameBufferInterface::
    AutoDelete( ) const
    {
      return m_auto_delete;
    }

    inline void FrameBufferInterface::
    AutoDelete( bool Value )
    {
      m_auto_delete = Value;
    }

    inline const std::string& FrameBufferInterface::
    BufferId( ) const
    {
      return m_stream_id;
    }

    inline void FrameBufferInterface::
    bufferId( const std::string& Id )
    {
      m_stream_id = Id;
    }

    template< class DT >
    inline void FrameBufferInterface::
    filter( const DT& Data )
    {
      filter( reinterpret_cast< const char_type* >( &Data ),
	      reinterpret_cast< const char_type* >( &Data + sizeof( Data ) ) );
    }
  } // namespace - Common
} // namespace - FrameCPP

#endif /* FrameCPP__COMMON__FRAME_BUFFER_INTERFACE_HH */
